import 'dart:convert';

/// utf8 to bytes
List utf8ToBytes(String text) {
  var bytes = [];

  /// utf8 encode
  var encoded = utf8.encode(text);

  /// for loop
  for (var i = 0; i < encoded.length; i++) {
    /// get string from char code
    var byte = String.fromCharCode(encoded[i]).codeUnitAt(0);

    /// add to list
    bytes.add(byte);
  }

  /// return list
  return bytes;
}

/// bin
int bin(x) {
  /// return int that is radix 2
  return int.parse(x, radix: 2);
}

/// get code points
List getCodePoints(String text) {
  var codePoints = [];

  var bytes = utf8ToBytes(text);

  /// for loop
  for (var i = 0; i < bytes.length;) {
    var codePoint;

    /// get list data
    var byte = bytes[i];

    /// if byte is 11110000
    if ((byte & bin("11110000")) == bin("11110000")) {
      /// Four byte UTF8
      var byte4 = bytes[i];
      var byte3 = bytes[i + 1];
      var byte2 = bytes[i + 2];
      var byte1 = bytes[i + 3];

      codePoint = ((byte4 & bin("00000111")) << 18) |
          ((byte3 & bin("00111111")) << 12) |
          ((byte2 & bin("00111111")) << 6) |
          (byte1 & bin("00111111"));

      i += 4;
    } else if ((byte & bin("11100000")) == bin("11100000")) {
      /// Three byte UTF8
      var byte3 = bytes[i];
      var byte2 = bytes[i + 1];
      var byte1 = bytes[i + 2];

      codePoint = ((byte3 & bin("00001111")) << 12) |
          ((byte2 & bin("00111111")) << 6) |
          (byte1 & bin("00111111"));

      i += 3;
    } else if ((byte & bin("11000000")) == bin("11000000")) {
      /// Two byte UTF8
      var byte2 = bytes[i];
      var byte1 = bytes[i + 1];

      codePoint = ((byte2 & bin("00011111")) << 6) | (byte1 & bin("00111111"));

      i += 2;
    } else if ((byte & 0x80) == 0) {
      /// Single byte UTF8
      codePoint = byte;

      i += 1;
    } else {
      throw new Error();
    }
    codePoints.add(codePoint);
  }

  return codePoints;
}
